@@ -28,7 +28,6 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <dlfcn.h>
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
@@ -52,12 +51,12 @@
 #include "map.h"
 #include "back-sch.h"
 #include "format.h"
+#include "back-sch-nss.h"
 
 static int
 bvstrprefix(const struct berval *bval, const char *s)
 {
 	size_t len;
-	int c;
 
 	len = strlen(s);
 	if (len < bval->bv_len) {
@@ -186,47 +185,6 @@ backend_search_filter_has_cn_uid(Slapi_Filter *filter, void *arg)
 	return SLAPI_FILTER_SCAN_CONTINUE;
 }
 
-static char *
-backend_build_dn(const char *attribute, const char *value,
-		 const char *container_sdn)
-{
-	Slapi_RDN *rdn;
-	Slapi_DN *sdn;
-	char *val, *dn = NULL;
-	const char *ndn, *hexchars = "0123456789ABCDEF";
-	int i;
-
-	val = malloc(strlen(value) * 3 + 1);
-	if (val == NULL) {
-		return NULL;
-	}
-	rdn = slapi_rdn_new();
-	if (rdn == NULL) {
-		free(val);
-		return NULL;
-	}
-        for (i = 0; value[i] != '\0'; i++) {
-		val[i * 3] = '\\';
-		val[i * 3 + 1] = hexchars[(value[i] & 0xf0) >> 4];
-		val[i * 3 + 2] = hexchars[value[i] & 0xf];
-	}
-	val[i * 3] = '\0';
-	if (slapi_rdn_add(rdn, attribute, val) == 1) {
-		sdn = slapi_sdn_new_dn_byval(container_sdn);
-		if (sdn != NULL) {
-			sdn = slapi_sdn_add_rdn(sdn, rdn);
-			ndn = slapi_sdn_get_ndn(sdn);
-			if (ndn != NULL) {
-				dn = slapi_ch_strdup(ndn);
-			}
-			slapi_sdn_free(&sdn);
-		}
-	}
-	free(val);
-	slapi_rdn_free(&rdn);
-	return dn;
-}
-
 static Slapi_Entry *
 backend_make_user_entry_from_nsswitch_passwd(struct passwd *pwd,
 					     char *container_sdn,
@@ -257,7 +215,7 @@ backend_make_user_entry_from_nsswitch_passwd(struct passwd *pwd,
 		return NULL;
 	}
 
-	dn = backend_build_dn("uid", name, container_sdn);
+	dn = format_build_dn("uid", name, container_sdn);
 	if (dn == NULL) {
 		slapi_log_error(SLAPI_LOG_FATAL,
 				cbdata->state->plugin_desc->spd_id,
@@ -335,143 +293,6 @@ backend_make_user_entry_from_nsswitch_passwd(struct passwd *pwd,
 	return entry;
 }
 
-/* Possible results of lookup using a nss_* function.
- * Note: don't include nss.h as its path gets overriden by NSS library */
-enum nss_status
-{
-  NSS_STATUS_TRYAGAIN = -2,
-  NSS_STATUS_UNAVAIL,
-  NSS_STATUS_NOTFOUND,
-  NSS_STATUS_SUCCESS,
-  NSS_STATUS_RETURN
-};
-
-struct nss_ops_ctx {
-	void *dl_handle;
-
-	enum nss_status (*getpwnam_r)(const char *name, struct passwd *result,
-			  char *buffer, size_t buflen, int *errnop);
-	enum nss_status (*getpwuid_r)(uid_t uid, struct passwd *result,
-			  char *buffer, size_t buflen, int *errnop);
-	enum nss_status (*setpwent)(void);
-	enum nss_status (*getpwent_r)(struct passwd *result,
-			  char *buffer, size_t buflen, int *errnop);
-	enum nss_status (*endpwent)(void);
-
-	enum nss_status (*getgrnam_r)(const char *name, struct group *result,
-			  char *buffer, size_t buflen, int *errnop);
-	enum nss_status (*getgrgid_r)(gid_t gid, struct group *result,
-			  char *buffer, size_t buflen, int *errnop);
-	enum nss_status (*setgrent)(void);
-	enum nss_status (*getgrent_r)(struct group *result,
-			  char *buffer, size_t buflen, int *errnop);
-	enum nss_status (*endgrent)(void);
-
-	enum nss_status (*initgroups_dyn)(const char *user, gid_t group,
-			      long int *start, long int *size,
-			      gid_t **groups, long int limit,
-			      int *errnop);
-};
-
-void backend_nss_init_context(struct nss_ops_ctx **nss_context)
-{
-	struct nss_ops_ctx *ctx = NULL;
-
-	if (nss_context == NULL) {
-		return;
-	}
-
-	ctx = calloc(1, sizeof(struct nss_ops_ctx));
-
-	*nss_context = ctx;
-	if (ctx == NULL) {
-		return;
-	}
-
-	ctx->dl_handle = dlopen("libnss_sss.so.2", RTLD_NOW);
-	if (ctx->dl_handle == NULL) {
-		goto fail;
-	}
-
-	ctx->getpwnam_r = dlsym(ctx->dl_handle, "_nss_sss_getpwnam_r");
-	if (ctx->getpwnam_r == NULL) {
-		goto fail;
-	}
-
-	ctx->getpwuid_r = dlsym(ctx->dl_handle, "_nss_sss_getpwuid_r");
-	if (ctx->getpwuid_r == NULL) {
-		goto fail;
-	}
-
-	ctx->setpwent = dlsym(ctx->dl_handle, "_nss_sss_setpwent");
-	if (ctx->setpwent == NULL) {
-		goto fail;
-	}
-
-	ctx->getpwent_r = dlsym(ctx->dl_handle, "_nss_sss_getpwent_r");
-	if (ctx->getpwent_r == NULL) {
-		goto fail;
-	}
-
-	ctx->endpwent = dlsym(ctx->dl_handle, "_nss_sss_endpwent");
-	if (ctx->endpwent == NULL) {
-		goto fail;
-	}
-
-	ctx->getgrnam_r = dlsym(ctx->dl_handle, "_nss_sss_getgrnam_r");
-	if (ctx->getgrnam_r == NULL) {
-		goto fail;
-	}
-
-	ctx->getgrgid_r = dlsym(ctx->dl_handle, "_nss_sss_getgrgid_r");
-	if (ctx->getgrgid_r == NULL) {
-		goto fail;
-	}
-
-	ctx->setgrent = dlsym(ctx->dl_handle, "_nss_sss_setgrent");
-	if (ctx->setgrent == NULL) {
-		goto fail;
-	}
-
-	ctx->getgrent_r = dlsym(ctx->dl_handle, "_nss_sss_getgrent_r");
-	if (ctx->getgrent_r == NULL) {
-		goto fail;
-	}
-
-	ctx->endgrent = dlsym(ctx->dl_handle, "_nss_sss_endgrent");
-	if (ctx->endgrent == NULL) {
-		goto fail;
-	}
-
-	ctx->initgroups_dyn = dlsym(ctx->dl_handle, "_nss_sss_initgroups_dyn");
-	if (ctx->initgroups_dyn == NULL) {
-		goto fail;
-	}
-
-	return;
-
-fail:
-	backend_nss_free_context(nss_context);
-
-	return;
-}
-
-void
-backend_nss_free_context(struct nss_ops_ctx **nss_context)
-{
-	if (nss_context == NULL) {
-		return;
-	}
-
-	if ((*nss_context)->dl_handle != NULL) {
-		dlclose((*nss_context)->dl_handle);
-	}
-
-	free((*nss_context));
-	*nss_context = NULL;
-}
-
-
 
 static Slapi_Entry **
 backend_retrieve_user_entry_from_nsswitch(char *user_name, bool_t is_uid,
@@ -497,13 +318,13 @@ repeat:
 	}
 
 	if (is_uid) {
-		rc = ctx->getpwuid_r((uid_t) atoll(user_name), &pwd,
-				     cbdata->nsswitch_buffer,
-				     cbdata->nsswitch_buffer_len, &lerrno);
+		rc = backend_nss_getpwuid(ctx, (uid_t) atoll(user_name), &pwd,
+					  cbdata->nsswitch_buffer,
+					  cbdata->nsswitch_buffer_len, &result, &lerrno);
 	} else {
-		rc = ctx->getpwnam_r(user_name, &pwd,
-				     cbdata->nsswitch_buffer,
-				     cbdata->nsswitch_buffer_len, &lerrno);
+		rc = backend_nss_getpwnam(ctx, user_name, &pwd,
+					  cbdata->nsswitch_buffer,
+					  cbdata->nsswitch_buffer_len, &result, &lerrno);
 	}
 
 	if ((rc != NSS_STATUS_SUCCESS)) {
@@ -556,7 +377,7 @@ backend_make_group_entry_from_nsswitch_group(struct group *grp,
 		return NULL;
 	}
 
-	dn = backend_build_dn("cn", grp->gr_name, container_sdn);
+	dn = format_build_dn("cn", grp->gr_name, container_sdn);
 	if (dn == NULL) {
 		slapi_log_error(SLAPI_LOG_FATAL,
 				cbdata->state->plugin_desc->spd_id,
@@ -632,13 +453,13 @@ repeat:
 	}
 
 	if (is_gid) {
-		rc = ctx->getgrgid_r((gid_t) atoll(group_name), &grp,
-				     cbdata->nsswitch_buffer,
-				     cbdata->nsswitch_buffer_len, &lerrno);
+		rc = backend_nss_getgrgid(ctx, (gid_t) atoll(group_name), &grp,
+					  cbdata->nsswitch_buffer,
+					  cbdata->nsswitch_buffer_len, &result, &lerrno);
 	} else {
-		rc = ctx->getgrnam_r(group_name, &grp,
-				     cbdata->nsswitch_buffer,
-				     cbdata->nsswitch_buffer_len, &lerrno);
+		rc = backend_nss_getgrnam(ctx, group_name, &grp,
+					  cbdata->nsswitch_buffer,
+					  cbdata->nsswitch_buffer_len, &result, &lerrno);
 	}
 	if ((rc != NSS_STATUS_SUCCESS)) {
 		if (lerrno == ERANGE) {
@@ -692,9 +513,9 @@ repeat:
 		return NULL;
 	}
 
-	rc = ctx->getgrgid_r(gid, &grp,
-			     cbdata->nsswitch_buffer,
-			     cbdata->nsswitch_buffer_len, &lerrno);
+	rc = backend_nss_getgrgid(ctx, gid, &grp,
+				  cbdata->nsswitch_buffer,
+				  cbdata->nsswitch_buffer_len, &result, &lerrno);
 
 	if ((rc != NSS_STATUS_SUCCESS)) {
 		if (lerrno == ERANGE) {
@@ -730,8 +551,7 @@ backend_retrieve_group_list_from_nsswitch(char *user_name, char *container_sdn,
 	int i, idx;
 	struct nss_ops_ctx *ctx = NULL;
 	int lerrno = 0;
-	long int ngroups = 0;
-	long int start = 0;
+	int ngroups = 0;
 	enum nss_status rc;
 
 	ctx = cbdata->state->nss_context;
@@ -743,9 +563,9 @@ repeat:
 		return NULL;
 	}
 
-	rc = ctx->getpwnam_r(user_name, &pwd,
-			     cbdata->nsswitch_buffer,
-			     cbdata->nsswitch_buffer_len, &lerrno);
+	rc = backend_nss_getpwnam(ctx, user_name, &pwd,
+				  cbdata->nsswitch_buffer,
+				  cbdata->nsswitch_buffer_len, &pwd_result, &lerrno);
 
 	if ((rc != NSS_STATUS_SUCCESS)) {
 		if (lerrno == ERANGE) {
@@ -764,19 +584,15 @@ repeat:
 	}
 
 	ngroups = 32;
-	start = 0;
 	grouplist = malloc(sizeof(gid_t) * ngroups);
 	if (grouplist == NULL) {
 		return NULL;
 	}
 
-	grouplist[0] = pwd.pw_gid;
-	start++;
-
 	do {
-		rc = ctx->initgroups_dyn(user_name, pwd.pw_gid,
-					 &start, &ngroups, &grouplist,
-					 -1, &lerrno);
+		rc = backend_nss_getgrouplist(ctx, user_name, pwd.pw_gid,
+					      grouplist, &ngroups,
+					      &lerrno);
 		if ((rc != NSS_STATUS_SUCCESS)) {
 			tmp_list = realloc(grouplist, ngroups * sizeof(gid_t));
 			if (tmp_list == NULL) {
@@ -856,7 +672,7 @@ void
 backend_search_nsswitch(struct backend_set_data *set_data,
 			struct backend_search_cbdata *cbdata)
 {
-	int result, rc;
+	int result;
 	struct backend_search_filter_config config =
 		{FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, NULL, NULL, NULL};
 	struct backend_staged_search *staged = NULL;
