@@ -33,6 +33,7 @@
 #ifdef HAVE_DIRSRV_SLAPI_PLUGIN_H
 #include <nspr.h>
 #include <plhash.h>
+#include <plstr.h>
 #include <nss.h>
 #include <dirsrv/slapi-plugin.h>
 #else
@@ -506,7 +507,7 @@ backend_set_process_external_members(Slapi_PBlock *pb,
 		/* This group must exist because it exists in the original tree
 		 * but as dirsrv was restarted, SSSD might still consider its domain offline. */
 		is_group_exists = backend_retrieve_from_nsswitch(&staged, &cbdata);
-		if (!is_group_exists) {
+		if (!is_group_exists && !slapi_is_shutting_down()) {
 			slapi_log_error(SLAPI_LOG_FATAL, plugin_id,
 					"group \"%s\" does not exist because SSSD is offline.\n",
 					staged.name);
@@ -2099,6 +2100,193 @@ backend_write_cb(Slapi_PBlock *pb, struct plugin_state *state)
 	return ret;
 }
 
+#ifdef USE_IPA_IDVIEWS
+/* Check if ID override applies to an entry in our map cache
+ * and remove the entry in case it does. */
+
+static bool_t
+backend__get_original_uid_and_ndn(Slapi_Entry *e,
+				  char **original_uid,
+				  const char **original_anchor)
+{
+	char **elem = NULL;
+	char *v = NULL;
+	char *ndn = NULL;
+	char *view = NULL;
+	int n_elem = 0;
+	int i = 0;
+
+	if (e == NULL) {
+		return FALSE;
+	}
+
+	elem = slapi_entry_attr_get_charray_ext(e, "objectClass", &n_elem);
+	if (elem == NULL) {
+		/* weird, objectClass should be existing */
+		return FALSE;
+	}
+
+	for (i=0; i < n_elem; i++) {
+		if (strncasecmp(elem[i], "ipaOverrideAnchor", 17) == 0) {
+			break;
+		}
+	}
+
+	slapi_ch_array_free(elem);
+	if (i == n_elem) {
+		/* This is not an override, bail out */
+		return FALSE;
+	}
+
+	ndn = slapi_entry_get_ndn(e);
+	if (ndn == NULL) {
+		return FALSE;
+	}
+
+	view = PL_strcasestr(ndn, "cn=Default Trust View,");
+	if (view == NULL || view == ndn) {
+		return FALSE;
+	}
+
+	/* This is an ID override, we need to search for a referenced ipaOriginalUid or cn in our maps */
+	v = slapi_entry_attr_get_charptr(e, "ipaOriginalUid");
+	if (v == NULL) {
+		v = slapi_entry_attr_get_charptr(e, "cn");
+		if (v == NULL) {
+			return FALSE;
+		}
+	}
+
+	*original_uid = v;
+	*original_anchor = ndn;
+	return TRUE;
+}
+
+bool_t
+backend_entry_evict_if_related(const char *group, const char *set, bool_t flag,
+			       void *shared_set_data,
+			       Slapi_PBlock *pb,
+			       Slapi_Entry *e)
+{
+	struct backend_set_data *set_data;
+	struct plugin_state *state = NULL;
+	char *id = NULL;
+	char *original_uid = NULL;
+	const char *original_anchor = NULL;
+	const char *rdn_attribute[] = {NULL, "uid=%s,%s,%s", "cn=%s,%s,%s"};
+	bool_t result = FALSE;
+
+	set_data = shared_set_data;
+
+	/* We only interested in NSSWITCH-capable maps */
+	if (set_data->check_nsswitch == SCH_NSSWITCH_NONE) {
+		return FALSE;
+	}
+
+	/* See if the entry pre modification had original UID */
+	if (!backend__get_original_uid_and_ndn(e,
+					       &original_uid,
+					       &original_anchor)) {
+		return FALSE;
+	}
+
+	id = slapi_ch_smprintf(rdn_attribute[set_data->check_nsswitch],
+			       original_uid, set, group);
+
+	if (id == NULL) {
+		slapi_ch_free_string(&original_uid);
+		return FALSE;
+	}
+
+	slapi_pblock_get(pb, SLAPI_PLUGIN_PRIVATE, &state);
+	result = map_data_check_entry(state, group, set, id);
+	if (result) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
+				"evicted entry %s due to changed content of ID override %s\n",
+				id, original_anchor);
+
+		/* An entry corresponding to our target is found, evict it */
+		map_data_unset_entry(state, group, set, id);
+
+		/* Signal to SSSD that this entry is not cached anymore */
+		switch(set_data->check_nsswitch) {
+                case SCH_NSSWITCH_USER:
+			backend_nss_evict_user(state->nss_context, original_uid);
+			break;
+		case SCH_NSSWITCH_GROUP:
+			backend_nss_evict_group(state->nss_context, original_uid);
+			break;
+		default:
+			break;
+		}
+	}
+
+	slapi_ch_free_string(&id);
+	slapi_ch_free_string(&original_uid);
+	return result;
+}
+
+bool_t
+backend_entry_is_add_related(const char *group, const char *set, bool_t flag,
+			     void *shared_set_data,
+			     Slapi_PBlock *pb,
+			     Slapi_Entry *e)
+{
+	return backend_entry_evict_if_related(group, set, flag, shared_set_data, pb, e);
+}
+
+
+bool_t
+backend_entry_is_modify_related(const char *group, const char *set, bool_t flag,
+				void *shared_set_data,
+				Slapi_PBlock *pb,
+				Slapi_Entry *e_pre,
+				Slapi_Entry *e_post)
+{
+	return backend_entry_evict_if_related(group, set, flag, shared_set_data, pb, e_pre);
+}
+
+bool_t
+backend_entry_is_delete_related(const char *group, const char *set, bool_t flag,
+				void *shared_set_data,
+				Slapi_PBlock *pb,
+				Slapi_Entry *e)
+{
+	return backend_entry_evict_if_related(group, set, flag, shared_set_data, pb, e);
+}
+
+#else
+
+bool_t
+backend_entry_is_add_related(const char *group, const char *set, bool_t flag,
+			     void *shared_set_data,
+			     Slapi_PBlock *pb,
+			     Slapi_Entry *e)
+{
+	return FALSE;
+}
+
+bool_t
+backend_entry_is_modify_related(const char *group, const char *set, bool_t flag,
+				void *shared_set_data,
+				Slapi_PBlock *pb,
+				Slapi_Entry *e_pre,
+				Slapi_Entry *e_post)
+{
+	return FALSE;
+}
+
+bool_t
+backend_entry_is_delete_related(const char *group, const char *set, bool_t flag,
+				void *shared_set_data,
+				Slapi_PBlock *pb,
+				Slapi_Entry *e)
+{
+	return FALSE;
+}
+
+#endif
+
 static int
 backend_pre_write_cb(Slapi_PBlock *pb)
 {
