@@ -1856,11 +1856,21 @@ backend_shr_add_entry_cb(const char *group, const char *set, bool_t secure,
 
 	/* If the entry doesn't match the set, skip it. */
 	if (!backend_shr_entry_matches_set(set_data, cbdata->pb, cbdata->e)) {
-		slapi_log_error(SLAPI_LOG_PLUGIN,
-				cbdata->state->plugin_desc->spd_id,
-				"entry \"%s\" does not belong in "
-				"\"%s\"/\"%s\"\n",
-				cbdata->ndn, group, set);
+		/* Give backend a chance to perform other operations on the add op.
+		 * For example, ID override addition in FreeIPA need to be noted to
+		 * evict AD user/group entries from a map cache while ID overrides
+		 * themselves aren't present in any map.
+		 * Note that we could do this as a part of backend_shr_entry_matches_set()
+		 * but it is better to isolate this operation in a separate call. */
+		if (!backend_entry_is_add_related(group, set, secure,
+						  set_data, cbdata->pb,
+						  cbdata->e)) {
+			slapi_log_error(SLAPI_LOG_PLUGIN,
+					cbdata->state->plugin_desc->spd_id,
+					"entry \"%s\" does not belong in "
+					"\"%s\"/\"%s\"\n",
+					cbdata->ndn, group, set);
+		}
 		return TRUE;
 	}
 
@@ -2010,13 +2020,24 @@ backend_shr_modify_entry_cb(const char *group, const char *set, bool_t flag,
 					   cbdata->e_post) &&
 	    !backend_shr_entry_matches_set(set_data, cbdata->pb,
 					   cbdata->e_pre)) {
-		slapi_log_error(SLAPI_LOG_PLUGIN,
-				cbdata->state->plugin_desc->spd_id,
-				"\"%s\" not in \"%s\"/\"%s\", "
-				"before or after modify\n",
-				cbdata->ndn,
-				set_data->group,
-				set_data->set);
+		/* Give backend a chance to perform other operations on the modify op.
+		 * For example, ID override updates in FreeIPA need to be noted to
+		 * evict AD user/group entries from a map cache while ID overrides
+		 * themselves aren't present in any map.
+		 * Note that we could do this as a part of backend_shr_entry_matches_set()
+		 * but it is better to isolate this operation in a separate call. */
+		if (!backend_entry_is_modify_related(group, set, flag,
+						     set_data, cbdata->pb,
+						     cbdata->e_pre,
+						     cbdata->e_post)) {
+			slapi_log_error(SLAPI_LOG_PLUGIN,
+					cbdata->state->plugin_desc->spd_id,
+					"\"%s\" not in \"%s\"/\"%s\", "
+					"before or after modify\n",
+					cbdata->ndn,
+					set_data->group,
+					set_data->set);
+		}
 		return TRUE;
 	}
 	if (set_data->skip_uninteresting_updates &&
@@ -2631,6 +2652,16 @@ backend_shr_delete_entry_cb(const char *group, const char *set, bool_t flag,
 				group, set, set_data->group, set_data->set,
 				cbdata->ndn);
 		map_data_unset_entry(cbdata->state, group, set, cbdata->ndn);
+	} else {
+		/* Give backend a chance to perform other operations on the delete op.
+		 * For example, ID override removal in FreeIPA need to be noted to
+		 * evict AD user/group entries from a map cache while ID overrides
+		 * themselves aren't present in any map.
+		 * Note that we could do this as a part of backend_shr_entry_matches_set()
+		 * but it is better to isolate this operation in a separate call. */
+		(void) backend_entry_is_delete_related(group, set, flag,
+						       set_data, cbdata->pb,
+						       cbdata->e);
 	}
 	return TRUE;
 }
